<?php

/**
 * Classe per interagire con l'API REST di gCloud Storage
 *
 * @version 1.1.0
 * @package gCloud_Storage
 */

/**
 * Dipendiamo dalla HTTP_Request2 di PEAR
 */
require_once 'HTTP/Request2.php';

/**
 * Definizione della classe principale
 * 
 * @author Andrea Brancatelli <abrancatelli@schema31.it>
 * @package gCloud_Storage
 */
class gCloud_Storage
{

	/**
	 * Attuale versione della libreria
	 *
	 * @var string
	 * @access public
	 */
	const VERSION = "gCloud_Storage 1.1.0 [Pear]";

	/**
	 * Repository su cui operare
	 *
	 * @var string 
	 * @access public
	 */
	var $repositoryName = "";

	/**
	 * Chiave di autenticazione
	 *
	 * @var string 
	 * @access public
	 */
	var $Authentication = "";

	/**
	 * gCloud Storage Base Path - hardcoded a "storage31.schema31.it"
	 *
	 * @var string 
	 * @access public
	 */
	var $Storage31Host = "storage31.schema31.it";

	/**
	 * LastError - l'ultimo errore ricevuto
	 *
	 * @var string 
	 * @access public
	 */
	var $LastError = "";


	private $putStatusCodeOK    = array(201);
	private $postStatusCodeOK   = array(201);
	private $getStatusCodeOK    = array(200);
	private $deleteStatusCodeOK = array(200);

	/**
	 * Carica un documento su gCloud Storage
	 *
	 * @param string $localFile Path del file da inviare su questo filesystem
	 * @param string $mime Mime part del file da inviare (default application/octet-stream)
	 * @param string $publicName Nome del file da inviare a gCloud Storage (di default prende il basename di $localFile)
	 * @param string $fileKey Riferimento ad una fileKey già esistente per effettuare la "sovrascrittura" di un file
	 * @return boolean|string torna la fileKey ritornata da gCloud Storage oppure FALSE in caso di errore
	 * @access public
	 */
	function sendFile($localFile, $mime = "application/octet-stream", $publicName = FALSE, $fileKey = FALSE) {

		/**
		 * Se non ci viene passato un publicName esplicito lo ricaviamo dal nome del file da inviare
		 */
		if ($publicName == FALSE) {
			$publicName = basename($localFile);
		}

		/**
		 * Verifico se c'è una fileKey preimpostata
		 */
		if ($fileKey != FALSE) {
			$extraUrl = "fileKey/".$fileKey;
		}
		else $extraUrl = "";

		/**
		 * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
		 */
		$request = new HTTP_Request2('http://'.$this->Storage31Host."/api/resource/".$extraUrl);
		$request->setMethod(HTTP_Request2::METHOD_POST)
			->setHeader('Authentication', $this->Authentication)
			->setHeader('repositoryName', $this->repositoryName)
			->setHeader('User-Agent', self::VERSION)
			->setHeader('Expect', "")
			->addUpload('somefile', $localFile, $publicName, $mime);

		/**
		 * Catturo la response della richiesta
		 */
		$HTTPRequest2Response = $request->send();
		$responseBody   = $HTTPRequest2Response->getBody();
		$responseStatus = $HTTPRequest2Response->getStatus();

		/**
		 * Mi assicuro che mi sia stato tornato un oggetto e che non contenga "error".
		 */
		if( ! in_array($responseStatus, $this->postStatusCodeOK) )
		{
			$fileKey = FALSE;
			$this->LastError = $responseBody;
		}
		else
		{
			$response = json_decode($responseBody);
			if   (!is_object($response)) $fileKey = FALSE;
			else $fileKey = $response->Result[0]->fileKey;
		}

		return $fileKey;
	}


	/**
	 * Torna le informazioni su un dato file
	 */
	function detailFile($fileKey, $fileVersion = FALSE) {

		/**
		 * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
		 */

		$url = 'http://'.$this->Storage31Host.'/api/resource/fileKey/'.$fileKey;
		if ($fileVersion != FALSE) $url .= '/fileVersion/'.$fileVersion;

		$request = new HTTP_Request2($url);
		$request->setConfig('follow_redirects', TRUE);
		$request->setConfig('max_redirects', 20);
		$request->setConfig('strict_redirects', TRUE);	
		$request->setMethod(HTTP_Request2::METHOD_GET)
			->setHeader('Authentication', $this->Authentication)
			->setHeader('repositoryName', $this->repositoryName)
			->setHeader('User-Agent', self::VERSION);

		/**
		 * Catturo la response della richiesta
		 */
		$HTTPRequest2Response = $request->send();
		$responseBody   = $HTTPRequest2Response->getBody();
		$responseStatus = $HTTPRequest2Response->getStatus();

		/**
		 * Mi assicuro che mi sia stato tornato un oggetto e che non contenga "error".
		 */
		if( ! in_array($responseStatus, $this->getStatusCodeOK) )
		{
			$this->LastError = $responseBody;
			return FALSE;
		}
		else
		{
			$response = json_decode($responseBody);
			if   (!is_object($response)) return FALSE;
			else return $response;
		}

	}


	/**
	 * Scarica un documento da gCloud Storage
	 */
	function getFile($fileKey, $fileVersion = FALSE) {

		$response = $this->detailFile($fileKey, $fileVersion);

		if ($response === FALSE) return FALSE;

		/**
		 * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
		 */

		$request = new HTTP_Request2($response->friendlyUrl);
		$request->setConfig('follow_redirects', TRUE);
		$request->setConfig('max_redirects', 20);
		$request->setConfig('strict_redirects', TRUE);	
		$request->setMethod(HTTP_Request2::METHOD_GET)
			->setHeader('Authentication', $this->Authentication)
			->setHeader('repositoryName', $this->repositoryName)
			->setHeader('User-Agent', self::VERSION);

		// CONTROLLO ERRORE:
		// $body = $request->send()->getBody();

		$HTTPRequest2Response = $request->send();
		$responseBody   = $HTTPRequest2Response->getBody();
		$responseStatus = $HTTPRequest2Response->getStatus();

		if( ! in_array($responseStatus, $this->getStatusCodeOK) )
		{
			$this->LastError = $responseBody;
			return FALSE;
		}
		else
		{
			return $responseBody;
		}		

	}


	/**
	 * Cancella un documento da gCloud Storage
	 */
	function deleteFile($fileKey, $fileVersion = FALSE) {

		/**
		 * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
		 */
		$url = 'http://'.$this->Storage31Host.'/api/resource/fileKey/'.$fileKey;
		if ($fileVersion != FALSE) $url .= '/fileVersion/'.$fileVersion;

		$request = new HTTP_Request2($url);		

		$request->setConfig('follow_redirects', TRUE);
		$request->setConfig('max_redirects', 20);
		$request->setConfig('strict_redirects', TRUE);	
		$request->setMethod(HTTP_Request2::METHOD_DELETE)
			->setHeader('Authentication', $this->Authentication)
			->setHeader('repositoryName', $this->repositoryName)
			->setHeader('User-Agent', self::VERSION);

		/**
		 * Catturo la response della richiesta
		 */
		$HTTPRequest2Response = $request->send();
		$responseBody   = $HTTPRequest2Response->getBody();
		$responseStatus = $HTTPRequest2Response->getStatus();

		/**
		 * Mi assicuro che mi sia stato tornato un oggetto e che non contenga "error".
		 * Questa cosa va fatta pi<9d> robusta.
		 * Comunque intanto ritorno FALSE oppure il fileKey in base alle circostanze
		 */
		//if (stristr($responseBody, "error")) 

		//TENTATIVO DI CONTROLLO PIU' ROBUSTO:
		if( ! in_array($responseStatus, $this->deleteStatusCodeOK) )
		{
			//$fileKey = FALSE;
			$this->LastError = $responseBody;
			return FALSE;
		}
		else
		{
			$response = json_decode($responseBody);
			if   (!is_object($response)) return FALSE;
			else return $response;
		}

	}

}
